from datetime import timedelta

from flask_jwt_extended import create_access_token

from portal.database.models import Course, User, Project
from tools.time import NOW, strip_seconds, portal_timezone
from . import utils
import json


def test_list(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    cpp_projects = len(cpp.projects)
    response = client.get(f'/courses/{cpp.codename}/projects/')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    projects = json.loads(str(response.get_data().decode("utf-8")))
    assert len(projects) == cpp_projects
    cpp_updated = Course.query.filter_by(codename="PB161").first()
    for p in projects:
        utils.assert_project_in(cpp_updated.projects, p)


def test_create(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    cpp_projects = len(cpp.projects)

    request_dict = {
        "name": "new_project",
        "config": {
            "test_files_source": "foo@git.git",
        },
    }
    request_json = json.dumps(request_dict)
    response = client.post(f"/courses/{cpp.codename}/projects/", data=request_json,
                           headers={"content-type": "application/json"})
    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    new_project = json.loads(str(response.get_data().decode("utf-8")))
    cpp_updated = Course.query.filter_by(codename="PB161").first()
    assert len(cpp_updated.projects) == cpp_projects + 1
    assert new_project['name'] == "new_project"
    assert new_project['id']
    assert new_project['config']['test_files_source'] == "foo@git.git"
    utils.assert_project_in(cpp_updated.projects, new_project)


def test_read(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    p = cpp.projects[0]
    response = client.get(f"/courses/{cpp.codename}/projects/{p.name}")
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    project = json.loads(str(response.get_data().decode("utf-8")))

    utils.assert_project(p, project)


def test_update(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    p = cpp.projects[0]
    p_name = p.name
    request_dict = dict(
        name="new project name",
    )
    request_json = json.dumps(request_dict)
    response = client.put(f"/courses/{cpp.codename}/projects/{p.name}", data=request_json,
                          headers={"content-type": "application/json"})
    assert response.status_code == 204
    assert response.mimetype == 'application/json'
    assert Project.query.filter(Project.course_id == cpp.id).filter(Project.name == "new project name").first()
    assert not Project.query.filter(Project.course_id == cpp.id).filter(Project.name == p_name).first()


def test_delete(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    p = cpp.projects[0]
    p_name = p.name
    cpp_projects = len(cpp.projects)
    response = client.delete(f"/courses/{cpp.codename}/projects/{p.name}")

    assert response.status_code == 204
    assert response.mimetype == 'application/json'
    cpp_updated = Course.query.filter_by(codename="PB161").first()
    assert len(cpp_updated.projects) == cpp_projects - 1
    assert not Project.query.filter(Project.course_id == cpp.id).filter(Project.name == p_name).first()


def test_config_read(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    p = cpp.projects[0]

    response = client.get(f"/courses/{cpp.codename}/projects/{p.name}/config")
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = json.loads(str(response.get_data().decode("utf-8")))
    utils.assert_project_config(p.config, data)


def test_config_update(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    p = cpp.projects[0]
    p_name = p.name

    new_time = NOW + timedelta(days=2)
    request_dict = dict(
        test_files_source="new source",
        pre_submit_script="a python script",
        submissions_allowed_to=new_time
    )
    request_json = json.dumps(request_dict, cls=utils.DateTimeEncoder)

    response = client.put(f"/courses/{cpp.codename}/projects/{p.name}/config", data=request_json,
                          headers={"content-type": "application/json"})
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    p_updated = Project.query.filter(Project.course_id == cpp.id).filter_by(name=p_name).first()
    assert p_updated.config.test_files_source == "new source"
    assert p_updated.config.pre_submit_script == "a python script"
    assert p_updated.config.submissions_allowed_to.replace(tzinfo=portal_timezone) == strip_seconds(new_time)


def test_list_submissions(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    p = cpp.projects[0]

    user = User.query.filter_by(username="xfoo").first()
    access_token = create_access_token(identity=user.username)
    response = client.get(f"/courses/{cpp.codename}/projects/{p.name}/submissions",
                          headers={"Authorization": f"Bearer {access_token}"})
    assert response.status_code == 200
    assert response.mimetype == 'application/json'
    submissions = json.loads(str(response.get_data().decode("utf-8")))
    assert len(submissions) == len(p.submissions)
    for s in submissions:
        utils.assert_submission_in(p.submissions, s)


def test_list_submissions_filter(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    p = cpp.projects[0]
    user = User.query.filter_by(username="xfoo").first()
    access_token = create_access_token(identity=user.username)
    response = client.get(f"/courses/{cpp.codename}/projects/{p.name}/submissions?user=xfoo",
                          headers={"Authorization": f"Bearer {access_token}"})
    assert response.status_code == 200
    assert response.mimetype == 'application/json'
    submissions = json.loads(str(response.get_data().decode("utf-8")))
    assert len(submissions) == len(p.submissions)
    for s in submissions:
        utils.assert_submission_in(p.submissions, s)
        assert s['user'] == user.id


def test_create_submission(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    p = cpp.projects[0]
    p_submissions = len(p.submissions)

    user = User.query.filter_by(username="xfoo").first()
    access_token = create_access_token(identity=user.username)
    request_dict = {
        "project_params": "data for Kontr",
        "file_params": {
            "source": {
                "type": "git",
                "url": "student@repo.git",
                "branch": "master",
                "checkout": "master"
            }
        }
    }
    request_json = json.dumps(request_dict, cls=utils.DateTimeEncoder)

    response = client.post(f"/courses/{cpp.codename}/projects/{p.name}/submissions", data=request_json,
                           headers={"Authorization": f"Bearer {access_token}", "content-type": "application/json"})

    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    new_submission = json.loads(str(response.get_data().decode("utf-8")))
    p_updated = Project.query.filter(Project.course_id == cpp.id).filter_by(name=p.name)
    user_updated = User.query.filter_by(username="xfoo").first()

    assert len(p_updated.submissions) == p_submissions + 1
    utils.assert_submission_in(p_updated.submissions, new_submission)
    utils.assert_submission_in(user_updated.submissions, new_submission)
    # TODO: @Peter, storage fail
