from portal.database.models import Course, Role, User
from . import utils
import json


def test_list(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    cpp_roles = len(cpp.roles)
    response = client.get(f'/courses/{cpp.codename}/roles/')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    roles = json.loads(str(response.get_data().decode("utf-8")))
    assert len(roles) == cpp_roles
    cpp_updated = Course.query.filter_by(codename="PB161").first()
    for r in roles:
        utils.assert_role_in(cpp_updated.roles, r)


def test_create(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    cpp_roles = len(cpp.roles)

    request_dict = {
        "name": "assistant",
        "description": "teacher assistant"
    }
    request_json = json.dumps(request_dict)
    response = client.post(f"/courses/{cpp.codename}/roles/", data=request_json,
                           headers={"content-type": "application/json"})
    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    new_role = json.loads(str(response.get_data().decode("utf-8")))
    cpp_updated = Course.query.filter_by(codename="PB161").first()
    assert len(cpp_updated.roles) == cpp_roles + 1
    assert new_role['name'] == "assistant"
    assert new_role['description'] == "teacher assistant"
    assert new_role['id']
    utils.assert_role_in(cpp_updated.roles, new_role)


def test_read(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    response = client.get(f"/courses/{cpp.codename}/roles/{r.name}")
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    role = json.loads(str(response.get_data().decode("utf-8")))

    utils.assert_role(r, role)


def test_update(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    r_name = r.name
    request_dict = dict(
        name="new role name",
        description="new role desc",
    )
    request_json = json.dumps(request_dict)
    response = client.put(f"/courses/{cpp.codename}/roles/{r.name}", data=request_json,
                          headers={"content-type": "application/json"})
    assert response.status_code == 204
    assert response.mimetype == 'application/json'
    r_updated = Role.query.filter(Role.course_id == cpp.id).filter(Role.name == "new role name").first()
    assert r_updated
    assert r_updated.description == "new role desc"
    assert not Role.query.filter(Role.course_id == cpp.id).filter(Role.name == r_name).first()


def test_delete(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    r_name = r.name
    cpp_roles = len(cpp.roles)
    response = client.delete(f"/courses/{cpp.codename}/roles/{r.name}")

    assert response.status_code == 204
    assert response.mimetype == 'application/json'
    cpp_updated = Course.query.filter_by(codename="PB161").first()
    assert len(cpp_updated.roles) == cpp_roles - 1
    assert not Role.query.filter(Role.course_id == cpp.id).filter(Role.name == r_name).first()


def test_users_list(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    response = client.get(f"/courses/{cpp.codename}/roles/{r.id}/users")

    assert response.status_code == 200
    assert response.mimetype == 'application/json'
    users = json.loads(str(response.get_data().decode("utf-8")))
    assert len(users) == len(r.users)
    for user in users:
        utils.assert_user_in(r.users, user)


def test_users_update_add(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    r_users = len(r.users)
    user = User.query.filter_by(username="xbar").first()
    assert user not in r.users

    request_dict = dict(
        add=[user.id]
    )
    request_json = json.dumps(request_dict)
    response = client.put(f"/courses/{cpp.codename}/roles/{r.name}/users", data=request_json,
                          headers={"content-type": "application/json"})

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == cpp.id).filter(Role.id == r.id).first()
    assert len(r_updated.users) == r_users + 1
    assert user in r_updated.users


def test_users_update_add_duplicate(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    r_users = len(r.users)
    user = r.users[0]
    assert user in r.users

    request_dict = dict(
        add=[user.id]
    )
    request_json = json.dumps(request_dict)
    response = client.put(f"/courses/{cpp.codename}/roles/{r.name}/users", data=request_json,
                          headers={"content-type": "application/json"})

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == cpp.id).filter(Role.id == r.id).first()
    assert len(r_updated.users) == r_users
    assert user in r_updated.users


def test_users_update_remove(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    r_users = len(r.users)
    user = r.users[0]
    assert user in r.users

    request_dict = dict(
        remove=[user.id]
    )
    request_json = json.dumps(request_dict)
    response = client.put(f"/courses/{cpp.codename}/roles/{r.name}/users", data=request_json,
                          headers={"content-type": "application/json"})

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == cpp.id).filter(Role.id == r.id).first()
    assert len(r_updated.users) == r_users - 1
    assert user not in r_updated.users


def test_users_update_remove_user_not_in(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    r_users = len(r.users)
    user = User.query.filter_by(username="xbar").first()
    assert user not in r.users

    request_dict = dict(
        remove=[user.id]
    )
    request_json = json.dumps(request_dict)
    response = client.put(f"/courses/{cpp.codename}/roles/{r.name}/users", data=request_json,
                          headers={"content-type": "application/json"})

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == cpp.id).filter(Role.id == r.id).first()
    assert len(r_updated.users) == r_users
    assert user not in r_updated.users


def test_add_user(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    role_users = len(r.users)
    u = User.query.filter_by(username="xbar").first()
    assert not u.groups

    response = client.put(f"/courses/{cpp.codename}/roles/{r.name}/users/{u.username}")
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == cpp.id).filter(Role.id == r.id).first()
    assert u in r_updated.users
    assert len(r_updated.users) == role_users + 1


def test_add_user_duplicate(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    role_users = len(r.users)
    u = r.users[0]
    assert u in r.users

    response = client.put(f"/courses/{cpp.codename}/roles/{r.name}/users/{u.username}")
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == cpp.id).filter(Role.id == r.id).first()
    assert u in r_updated.users
    assert len(r_updated.users) == role_users


def test_remove_user(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    role_users = len(r.users)
    u = r.users[0]

    response = client.delete(f"/courses/{cpp.codename}/roles/{r.name}/users/{u.username}")
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == cpp.id).filter(Role.id == r.id).first()
    assert u not in r_updated.users
    assert len(r_updated.users) == role_users - 1


def test_remove_user_not_in(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    role_users = len(r.users)
    u = User.query.filter_by(username="xbar").first()
    assert not u.groups

    response = client.delete(f"/courses/{cpp.codename}/roles/{r.name}/users/{u.username}")
    assert response.status_code == 400
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == cpp.id).filter(Role.id == r.id).first()
    assert u not in r_updated.users
    assert len(r_updated.users) == role_users


def test_permissions_list(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]

    response = client.get(f"/courses/{cpp.codename}/roles/{r.name}/permissions")
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = json.loads(str(response.get_data().decode("utf-8")))
    utils.assert_role_permissions(r.permissions, data)


def test_permissions_update(client):
    cpp = Course.query.filter_by(codename="PB161").first()
    r = cpp.roles[0]
    r_name = r.name

    request_dict = dict(
        viewCourseFull=True,
        writeRole=True,
        readAllSubmissions=True
    )
    request_json = json.dumps(request_dict, cls=utils.DateTimeEncoder)

    response = client.put(f"/courses/{cpp.codename}/roles/{r.name}/permissions", data=request_json,
                          headers={"content-type": "application/json"})
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == cpp.id).filter_by(name=r_name).first()
    assert r_updated.permissions.viewCourseFull
    assert r_updated.permissions.writeRole
    assert r_updated.permissions.readAllSubmissions


